@@ -21,6 +21,7 @@ Format String 2D array
 ['i']
 """
 
+import itertools
 import sys
 import logging
 
@@ -30,8 +31,8 @@ from .formatstring import FmtStr
 
 from typing import (
     Any,
+    Optional,
     Union,
-    Text,
     List,
     Sequence,
     overload,
@@ -40,77 +41,45 @@ from typing import (
     no_type_check,
 )
 
-actualize = str
 logger = logging.getLogger(__name__)
 
 # TODO check that strings used in arrays don't have tabs or spaces in them!
 
 
-def slicesize(s):
-    # type: (slice) -> int
+def slicesize(s: slice) -> int:
     return int((s.stop - s.start) / (s.step if s.step else 1))
 
 
-def fsarray(strings, *args, **kwargs):
-    # type: (List[Union[FmtStr, Text]], *Any, **Any) -> FSArray
-    """fsarray(list_of_FmtStrs_or_strings, width=None) -> FSArray
-
-    Returns a new FSArray of width of the maximum size of the provided
-    strings, or width provided, and height of the number of strings provided.
-    If a width is provided, raises a ValueError if any of the strings
-    are of length greater than this width"""
-
-    strings = list(strings)
-    if "width" in kwargs:
-        width = kwargs["width"]
-        del kwargs["width"]
-        if strings and any(len(s) > width for s in strings):
-            raise ValueError(f"Those strings won't fit for width {width}")
-    else:
-        width = max(len(s) for s in strings) if strings else 0
-    fstrings = [
-        s if isinstance(s, FmtStr) else fmtstr(s, *args, **kwargs) for s in strings
-    ]
-    arr = FSArray(len(fstrings), width, *args, **kwargs)
-    rows = [
-        fs.setslice_with_length(0, len(s), s, width)
-        for fs, s in zip(arr.rows, fstrings)
-    ]
-    arr.rows = rows
-    return arr
-
-
 class FSArray(Sequence):
     """A 2D array of colored text.
 
     Internally represented by a list of FmtStrs of identical size."""
 
     # TODO add constructor that takes fmtstrs instead of dims
-    def __init__(self, num_rows, num_columns, *args, **kwargs):
-        # type: (int, int, *Any, **Any) -> None
+    def __init__(
+        self, num_rows: int, num_columns: int, *args: Any, **kwargs: Any
+    ) -> None:
         self.saved_args, self.saved_kwargs = args, kwargs
-        self.rows = [
-            fmtstr("", *args, **kwargs) for _ in range(num_rows)
-        ]  # type: List[FmtStr]
+        self.rows: List[FmtStr] = [fmtstr("", *args, **kwargs) for _ in range(num_rows)]
         self.num_columns = num_columns
 
     @overload
-    def __getitem__(self, slicetuple):
-        # type: (int) -> FmtStr
+    def __getitem__(self, slicetuple: int) -> FmtStr:
         pass
 
     @overload
-    def __getitem__(self, slicetuple):
-        # type: (slice) -> List[FmtStr]
+    def __getitem__(self, slicetuple: slice) -> List[FmtStr]:
         pass
 
     @overload
-    def __getitem__(self, slicetuple):
-        # type: (Tuple[Union[slice, int], Union[slice, int]]) -> List[FmtStr]
+    def __getitem__(
+        self, slicetuple: Tuple[Union[slice, int], Union[slice, int]]
+    ) -> List[FmtStr]:
         pass
 
-    def __getitem__(self, slicetuple):
-        # type: (Union[int, slice, Tuple[Union[int, slice], Union[int, slice]]]) -> Union[FmtStr, List[FmtStr]]
+    def __getitem__(
+        self, slicetuple: Union[int, slice, Tuple[Union[int, slice], Union[int, slice]]]
+    ) -> Union[FmtStr, List[FmtStr]]:
         if isinstance(slicetuple, int):
             if slicetuple < 0:
                 slicetuple = len(self.rows) - slicetuple
@@ -120,34 +89,27 @@ class FSArray(Sequence):
         if isinstance(slicetuple, slice):
             rowslice = normalize_slice(len(self.rows), slicetuple)
             return self.rows[rowslice]
-        (
-            row_slice_or_int,
-            col_slice_or_int,
-        ) = slicetuple  # type: Tuple[Union[int, slice], Union[int, slice]]
+        row_slice_or_int, col_slice_or_int = slicetuple
         rowslice = normalize_slice(len(self.rows), row_slice_or_int)
         colslice = normalize_slice(self.num_columns, col_slice_or_int)
         # TODO clean up slices
         return [fs[colslice] for fs in self.rows[rowslice]]
 
-    def __len__(self):
-        # type: () -> int
+    def __len__(self) -> int:
         return len(self.rows)
 
     @property
-    def shape(self):
-        # type: () -> Tuple[int, int]
+    def shape(self) -> Tuple[int, int]:
         """Tuple of (len(rows, len(num_columns)) numpy-style shape"""
         return len(self.rows), self.num_columns
 
     @property
-    def height(self):
-        # type: () -> int
+    def height(self) -> int:
         """The number of rows"""
         return len(self.rows)
 
     @property
-    def width(self):
-        # type: () -> int
+    def width(self) -> int:
         """The number of columns"""
         return self.num_columns
 
@@ -158,7 +120,7 @@ class FSArray(Sequence):
         logger.debug("slice: %r", slicetuple)
         if isinstance(slicetuple, slice):
             rowslice, colslice = slicetuple, slice(None)
-            if isinstance(value, (bytes, str)):
+            if isinstance(value, str):
                 raise ValueError(
                     "if slice is 2D, value must be 2D as in of list type []"
                 )
@@ -237,28 +199,24 @@ class FSArray(Sequence):
             + self.rows[rowslice.stop :]
         )
 
-    def dumb_display(self):
-        # type: () -> None
+    def dumb_display(self) -> None:
         """Prints each row followed by a newline without regard for the terminal window size"""
         for line in self.rows:
             print(line)
 
     @classmethod
-    def diff(cls, a, b, ignore_formatting=False):
-        # type: (FSArray, FSArray, bool) -> Text
+    def diff(cls, a: "FSArray", b: "FSArray", ignore_formatting: bool = False) -> str:
         """Returns two FSArrays with differences underlined"""
 
-        def underline(x):
-            # type: (Text) -> Text
+        def underline(x: str) -> str:
             return f"\x1b[4m{x}\x1b[0m"
 
-        def blink(x):
-            # type: (Text) -> Text
+        def blink(x: str) -> str:
             return f"\x1b[5m{x}\x1b[0m"
 
         a_rows = []
         b_rows = []
-        max_width = max([len(row) for row in a] + [len(row) for row in b])
+        max_width = max(len(row) for row in itertools.chain(a, b))
         a_lengths = []
         b_lengths = []
         for a_row, b_row in zip(a, b):
@@ -276,29 +234,60 @@ class FSArray(Sequence):
                     a_char_for_eval = a_char
                     b_char_for_eval = b_char
                 if a_char_for_eval == b_char_for_eval:
-                    a_line += actualize(a_char)
-                    b_line += actualize(b_char)
+                    a_line += str(a_char)
+                    b_line += str(b_char)
                 else:
-                    a_line += underline(blink(actualize(a_char)))
-                    b_line += underline(blink(actualize(b_char)))
+                    a_line += underline(blink(str(a_char)))
+                    b_line += underline(blink(str(b_char)))
             a_rows.append(a_line)
             b_rows.append(b_line)
-        hdiff = "\n".join(
-            a_line + " %3d | %3d " % (a_len, b_len) + b_line
+        return "\n".join(
+            f"{a_line} {a_len:3d} | {b_len:3d} {b_line}"
             for a_line, b_line, a_len, b_len in zip(
                 a_rows, b_rows, a_lengths, b_lengths
             )
         )
-        return hdiff
 
 
-def simple_format(x):
-    # type: (Union[FSArray, List[FmtStr]]) -> Text
-    return "\n".join(actualize(l) for l in x)
+def fsarray(
+    strings: Sequence[Union[FmtStr, str]],
+    width: Optional[int] = None,
+    *args: Any,
+    **kwargs: Any,
+) -> FSArray:
+    """fsarray(list_of_FmtStrs_or_strings, width=None) -> FSArray
+
+    Returns a new FSArray of width of the maximum size of the provided
+    strings, or width provided, and height of the number of strings provided.
+    If a width is provided, raises a ValueError if any of the strings
+    are of length greater than this width"""
+
+    strings = list(strings)
+    if width is not None:
+        if strings and any(len(s) > width for s in strings):
+            raise ValueError(f"Those strings won't fit for width {width}")
+    else:
+        width = max(len(s) for s in strings) if strings else 0
+    arr = FSArray(len(strings), width, *args, **kwargs)
+    rows = [
+        fs.setslice_with_length(0, len(s), s, width)
+        for fs, s in zip(
+            arr.rows,
+            (
+                s if isinstance(s, FmtStr) else fmtstr(s, *args, **kwargs)
+                for s in strings
+            ),
+        )
+    ]
+    arr.rows = rows
+    return arr
 
 
-def assertFSArraysEqual(a, b):
-    # type: (FSArray, FSArray) -> None
+def simple_format(x: Union[FSArray, Sequence[FmtStr]]) -> str:
+    return "\n".join(str(l) for l in x)
+
+
+def assertFSArraysEqual(a: FSArray, b: FSArray) -> None:
     assert isinstance(a, FSArray)
     assert isinstance(b, FSArray)
     assert (
@@ -310,8 +299,7 @@ def assertFSArraysEqual(a, b):
         )
 
 
-def assertFSArraysEqualIgnoringFormatting(a, b):
-    # type: (FSArray, FSArray) -> None
+def assertFSArraysEqualIgnoringFormatting(a: FSArray, b: FSArray) -> None:
     """Also accepts arrays of strings"""
     assert len(a) == len(b), "fsarray heights do not match: %s %s \n%s \n%s" % (
         len(a),
